package com.chelvc.framework.rocketmq.fallback;

import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.kafka.context.KafkaContextHolder;
import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.message.Message;
import org.apache.rocketmq.client.apis.producer.Producer;
import org.apache.rocketmq.client.apis.producer.Transaction;

/**
 * RocketMQ降级Kafka消息生产者实现
 *
 * @author Woody
 * @date 2024/7/13
 */
@Slf4j
public class RocketMQKafkaProducer extends RocketMQFallbackProducer {
    public RocketMQKafkaProducer(Producer delegate) {
        super(delegate);
    }

    @Override
    protected void fallback(Message message, Transaction transaction) {
        if (log.isDebugEnabled()) {
            String topic = RocketMQContextHolder.topic(message), body = RocketMQContextHolder.body(message);
            log.debug("RocketMQ fallback kafka message: {}, {}", topic, body);
        }
        String topic = RocketMQContextHolder.topic(message, '-');
        Object payload = JacksonUtils.deserialize(RocketMQContextHolder.body(message));
        String ordering = message.getMessageGroup().orElse(null);
        Session session = JacksonUtils.deserialize(message.getProperties().get(Session.NAMING), Session.class);
        SessionContextHolder.setSession(session);
        try {
            if (ordering == null) {
                KafkaContextHolder.send(topic, payload);
            } else {
                KafkaContextHolder.send(topic, payload, ordering);
            }
        } finally {
            SessionContextHolder.clearSessionContext();
        }
    }
}
